#exec OBJ LOAD FILE=..\staticmeshes\UR-Cars.usx
#exec OBJ LOAD FILE=..\sounds\WeaponSounds.uax
#exec OBJ LOAD FILE=..\textures\VehicleFX.utx
#exec OBJ LOAD FILE=..\textures\UR-Nas_tex.utx
#exec OBJ LOAD FILE=..\UnrealRacing\Sounds\URSounds.uax

class Nascar extends URCar
    placeable;

// triggers used to get into the Nascar
var const vector	FrontTriggerOffset;
var URCarTrigger	FLTrigger, FRTrigger, FlipTrigger;


// headlight projector
var const vector	HeadlightOffset;
var URCarHeadlight	Headlight;
var bool		HeadlightOn;

// Light materials
var Material			ReverseMaterial;
var Material			BrakeMaterial;
var Material			TailOffMaterial;

var Material			HeadlightOnMaterial;
var Material			HeadlightOffMaterial;

var URCarHeadlightCorona HeadlightCorona[2];
var (URCar) vector	HeadlightCoronaOffset[2];

// URCar sound things
var (URCar) float	EnginePitchScale;
var (URCar) sound	URCarStartSound;
var (URCar) sound	URCarIdleSound;
var (URCar) float	HitSoundThreshold;
var (URCar) sound	URCarSquealSound;
var (URCar) float	SquealVelThresh;
var (URCar) sound	URCarHitSound;

// Used to prevent triggering sounds continuously
var float			UntilNextImpact;

// Distance below centre of tire to place dust.
var float			DustDrop;

// Ratio between dust kicked up and amount wheels are slipping
var (URCar) float	DustSlipRate;
var (URCar) float 	DustSlipThresh;

// Maximum speed at which you can get in the vehicle.
var (URCar) float		TriggerSpeedThresh;
var bool			TriggerState; // true for on, false for off.
var bool			FlipTriggerState;

// Destroyed Buggy
var (URCar) class<Actor>	DestroyedEffect;
var (URCar) sound		DestroyedSound;

var (URCar) String URCarStartForce;
var (URCar) String URCarIdleForce;
var (URCar) String URCarSquealForce;
var (URCar) String URCarHitForce;

replication
{
	reliable if(Role == ROLE_Authority)
		HeadlightOn;
}

function PreBeginPlay()
{
	Super.PreBeginPlay();
	if ( Level.IsDemoBuild() )
		Destroy();
}

simulated function PostNetBeginPlay()
{
	local vector RotX, RotY, RotZ;
	local int i;

    Super.PostNetBeginPlay();

    GetAxes(Rotation,RotX,RotY,RotZ);

	// Only have triggers on server
	if(Level.NetMode != NM_Client)
	{
		// Create triggers for gettting into the URCar
		FLTrigger = spawn(class'URCarTrigger', self,, Location + FrontTriggerOffset.X * RotX + FrontTriggerOffset.Y * RotY + FrontTriggerOffset.Z * RotZ);
		FLTrigger.SetBase(self);
		FLTrigger.SetCollision(true, false, false);

		FRTrigger = spawn(class'URCarTrigger', self,, Location + FrontTriggerOffset.X * RotX - FrontTriggerOffset.Y * RotY + FrontTriggerOffset.Z * RotZ);
		FRTrigger.SetBase(self);
		FRTrigger.SetCollision(true, false, false);

		TriggerState = true;

		// Create trigger for flipping the bulldog back over.
		FlipTrigger = spawn(class'URCarTrigger', self,, Location);
		Fliptrigger.bCarFlipTrigger = true;
		FlipTrigger.SetBase(self);
		FlipTrigger.SetCollisionSize(200, 120);
		FlipTrigger.SetCollision(false, false, false);

		FlipTriggerState = false;
	}

	// Dont bother making emitters etc. on dedicated server
	if(Level.NetMode != NM_DedicatedServer)
	{
		// Create headlight projector. We make sure it doesn't project on the vehicle itself.
		Headlight = spawn(class'URCarHeadlight', self,, Location + HeadlightOffset.X * RotX + HeadlightOffset.Y * RotY + HeadlightOffset.Z * RotZ);
		Headlight.SetBase(self);
		Headlight.SetRelativeRotation(rot(-2000, 32768, 0));

		// Create wheel dust emitters.
		Dust[0] = spawn(class'URCarDust', self,, Location + WheelFrontAlong*RotX + WheelFrontAcross*RotY + (WheelVert-DustDrop)*RotZ);
		Dust[0].SetBase(self);

		Dust[1] = spawn(class'URCarDust', self,, Location + WheelFrontAlong*RotX - WheelFrontAcross*RotY + (WheelVert-DustDrop)*RotZ);
		Dust[1].SetBase(self);	

		Dust[2] = spawn(class'URCarDust', self,, Location + WheelRearAlong*RotX + WheelRearAcross*RotY + (WheelVert-DustDrop)*RotZ);
		Dust[2].SetBase(self);

		Dust[3] = spawn(class'URCarDust', self,, Location + WheelRearAlong*RotX - WheelRearAcross*RotY + (WheelVert-DustDrop)*RotZ);
		Dust[3].SetBase(self);

		// Set light materials
		Skins[1]=BrakeMaterial;
		Skins[2]=HeadlightOffMaterial;

		// Spawn headlight coronas
		for(i=0; i<2; i++)
		{
			HeadlightCorona[i] = spawn(class'URCarHeadlightCorona', self,, Location + (HeadlightCoronaOffset[i] >> Rotation) );
			HeadlightCorona[i].SetBase(self);
		}
	}

    // For KImpact event
    KSetImpactThreshold(HitSoundThreshold);

	// If this is not 'authority' version - don't destroy it if there is a problem. 
	// The network should sort things out.
	if(Role != ROLE_Authority)
	{
		KarmaParams(KParams).bDestroyOnSimError = False;
		KarmaParams(frontLeft.KParams).bDestroyOnSimError = False;
		KarmaParams(frontRight.KParams).bDestroyOnSimError = False;
		KarmaParams(rearLeft.KParams).bDestroyOnSimError = False;
		KarmaParams(rearRight.KParams).bDestroyOnSimError = False;
	}
}

simulated event Destroyed()
{
	local int i;

	//Log("URCar Destroyed");

	// Clean up random stuff attached to the car
	if(Level.NetMode != NM_Client)
	{

		FLTrigger.Destroy();
		FRTrigger.Destroy();
		FlipTrigger.Destroy();
	}

	if(Level.NetMode != NM_DedicatedServer)
	{
		Headlight.Destroy();

		for(i=0; i<4; i++)
			Dust[i].Destroy();

		for(i=0; i<2; i++)
			HeadlightCorona[i].Destroy();
	}

	// This will destroy wheels & joints
	Super.Destroyed();

	// Trigger destroyed sound and effect
	if(Level.NetMode != NM_DedicatedServer)
	{
		spawn(DestroyedEffect, self, , Location );
		PlaySound(DestroyedSound);
	}
}

function KImpact(actor other, vector pos, vector impactVel, vector impactNorm)
{
    /* Only make noises for URCar-world impacts */
    if(other != None)
        return;

    if(UntilNextImpact < 0)
    {
        PlaySound(URCarHitSound);

        // hack to stop the sound repeating rapidly on impacts
        UntilNextImpact = GetSoundDuration(URCarHitSound);
        
        
    }
}

simulated function ClientKDriverEnter(PlayerController pc)
{
	DriverCompName = level.ComputerName;
	Super.ClientKDriverEnter(pc);
}

function KDriverEnter(Pawn p)
{
	Super.KDriverEnter(p);

	PlaySound(URCarStartSound);

	AmbientSound=URCarIdleSound;

	p.bHidden = True;
	ReceiveLocalizedMessage(class'UnrealRacing.URCarMessage', 1);
}

simulated function ClientKDriverLeave(PlayerController pc)
{
	//Log("URCar ClientKDriverLeave");

	//StartFlip(self); //don't get out, just make it the right way up. Maybe need a fix so you can't cheat with it? Could be handy on corners. Doesn't seem to work anyway.

	DriverCompName = "No Driver";
	//Super.ClientKDriverLeave(pc);
}

function bool KDriverLeave(bool bForceLeave)
{
	/*local Pawn OldDriver;

	//Log("URCar KDriverLeave");

	OldDriver = Driver;

	// If we succesfully got out of the car, make driver visible again.
	if( Super.KDriverLeave(bForceLeave) )
	{
		OldDriver.bHidden = false;
		AmbientSound = None;	
		return true;
	}
	else
		return false;*/
}

simulated function Tick(float Delta)
{
    Local float TotalSlip, EnginePitch, VMag;
	local int i;

    Super.Tick(Delta);

	if(Role == ROLE_Authority)
	{
		// Only put headlights on when driver is in. Save batteries!
		if(Driver != None)
			HeadlightOn=true;
		else
			HeadlightOn=false;

	}

	// Dont bother doing effects on dedicated server.
	if(Level.NetMode != NM_DedicatedServer)
	{
		// Update brake light colors.
		if(Gear == -1 && OutputBrake == false)
		{
			Skins[1] = ReverseMaterial;
		}
		else if(Gear == 0 && OutputBrake == true)
		{
			Skins[1] = BrakeMaterial;
		}
		else
		{
			Skins[1] = TailOffMaterial;
		}

		// Set headlight projector state.
		Headlight.DetachProjector();
		if(HeadlightOn)
		{
			Headlight.AttachProjector();
			Skins[2] = HeadlightOnMaterial;
			for(i=0; i<2; i++)
				HeadlightCorona[i].bHidden = false;
		}
		else
		{
			Skins[2] = HeadlightOffMaterial;
			for(i=0; i<2; i++)
				HeadlightCorona[i].bHidden = true;
		}

		// Update dust kicked up by wheels.
		Dust[0].UpdateDust(frontLeft, DustSlipRate, DustSlipThresh);
		Dust[1].UpdateDust(frontRight, DustSlipRate, DustSlipThresh);
		Dust[2].UpdateDust(rearLeft, DustSlipRate, DustSlipThresh);
		Dust[3].UpdateDust(rearRight, DustSlipRate, DustSlipThresh);
	}

    UntilNextImpact -= Delta;

	// This assume the sound is an idle-ing sound, and increases with pitch
	// as wheels speed up.
	EnginePitch = 32 + ((getRPM()/EnginePitchScale) * (255-64));
	if (gear==0) //quick fix, was at full revs while idling
	{
		EnginePitch = 32;
	}
    SoundPitch = FClamp(EnginePitch, 0, 254);
	//Log("Engine Pitch:"$SoundPitch);
	if (Gear>=0)
	{
		if (throttle>=0)
		{
			SoundVolume = throttle*128+127;
		}
		else
		{
			SoundVolume = 128;
		}
	}
	else
	{
		if (throttle<=0)
		{
			SoundVolume = -throttle*128+127;
		}
		else
		{
			SoundVolume = 128;
		}
	}
    // Currently, just use rear wheels for slipping noise
    TotalSlip = rearLeft.GroundSlipVel + rearRight.GroundSlipVel;
    //log("TotalSlip:"$TotalSlip);
	if(TotalSlip > SquealVelThresh)
	{
		rearLeft.SoundVolume = rearLeft.GroundSlipVel*10;
		rearLeft.AmbientSound = URCarSquealSound;
		rearRight.SoundVolume = rearRight.GroundSlipVel*10;
		rearRight.AmbientSound = URCarSquealSound;
	}
	else
	{
		rearLeft.AmbientSound = None;
		rearRight.AmbientSound = None;
	}

	// Dont have triggers on network clients.
	if(Level.NetMode != NM_Client)
	{
		// If vehicle is moving, disable collision for trigger.
		VMag = VSize(Velocity);

		if(!bIsInverted && VMag < TriggerSpeedThresh && TriggerState == false)
		{
			FLTrigger.SetCollision(true, false, false);
			FRTrigger.SetCollision(true, false, false);
			TriggerState = true;
		}
		else if((bIsInverted || VMag > TriggerSpeedThresh) && TriggerState == true)
		{
			FLTrigger.SetCollision(false, false, false);
			FRTrigger.SetCollision(false, false, false);
			TriggerState = false;
		}

		// If vehicle is inverted, and slow, turn on big 'flip vehicle' trigger.
		if(bIsInverted && VMag < TriggerSpeedThresh && FlipTriggerState == false)
		{
			FlipTrigger.SetCollision(true, false, false);
			FlipTriggerState = true;	
		}
		else if((!bIsInverted || VMag > TriggerSpeedThresh) && FlipTriggerState == true)
		{
			FlipTrigger.SetCollision(false, false, false);
			FlipTriggerState = false;
		}
	}
}

// Really simple at the moment!
function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	// Avoid damage healing the car!
	if(Damage < 0)
		return;

	if(damageType == class'DamTypeSuperShockBeam')
		Health -= 100; // Instagib doesn't work on vehicles
	else
		Health -= 0.5 * Damage; // Weapons do less damage

	// The vehicle is dead!
	if(Health <= 0)
	{
		if ( Controller != None )
		{
			if( Controller.bIsPlayer )
			{
				ClientKDriverLeave(PlayerController(Controller)); // Just to reset HUD etc.
				Controller.PawnDied(self); // This should unpossess the controller and let the player respawn
			}
			else
				Controller.Destroy();
		}

		Destroy(); // Destroy the vehicle itself (see Destroyed below)
	}

    //KAddImpulse(momentum, hitlocation);
}

defaultproperties
{
	DrawType=DT_StaticMesh
	StaticMesh=StaticMesh'UR-Cars.Cars.Nascar'

     Begin Object Class=KarmaParamsRBFull Name=KParams0
        KInertiaTensor(0)=5
        KInertiaTensor(1)=0
        KInertiaTensor(2)=0
        KInertiaTensor(3)=6.5
        KInertiaTensor(4)=0
        KInertiaTensor(5)=6
		KCOMOffset=(X=-0.25,Y=0.0,Z=-0.2)
		KStartEnabled=True
		KFriction=0.75
		KLinearDamping=0.0
		KAngularDamping=0.0
		bKNonSphericalInertia=False
        bHighDetailOnly=False
        bClientOnly=False
		bKDoubleTickRate=True
        Name="KParams0"
    End Object
    KParams=KarmaParams'KParams0'

	DrawScale=1.0

	FrontTireClass=class'NasTire'
	RearTireClass=class'NasTire'

	WheelFrontAlong=-72
	WheelFrontAcross=40
	WheelRearAlong=72
	WheelRearAcross=40
	WheelVert=-4

	DestroyedEffect=class'XEffects.RocketExplosion'
	DestroyedSound=sound'WeaponSounds.P1RocketLauncherAltFire'

	FrontTriggerOffset=(X=0,Y=75,Z=10)

	TriggerSpeedThresh=40

	FlipTorque=75
	FlipTime=3

	HeadlightOffset=(X=-100,Y=0,Z=25)
	HeadlightOn=false

	// Light materials
	ReverseMaterial=Material'UR-CarTex.Porsche.por_rev'
	BrakeMaterial=Material'UR-CarTex.Porsche.por_brk'
	TailOffMaterial=Material'UR-CarTex.Porsche.por_rear'

	HeadlightOnMaterial=Material'UR-CarTex.Porsche.por_head_ON'
	HeadlightOffMaterial=Material'UR-CarTex.Porsche.por_head_OFF'

	HeadlightCoronaOffset(0)=(X=-112,Y=32,Z=8)
	HeadlightCoronaOffset(1)=(X=-112,Y=-32,Z=8)

	// Sounds
	EnginePitchScale=855
	HitSoundThreshold=1
	SquealVelThresh=1
	URCarStartSound=sound'URSounds.cars.startup'
	URCarIdleSound=sound'URSounds.cars.Idle'
	URCarHitSound=sound'URSounds.cars.Crash'
	URCarSquealSound=sound'URSounds.cars.skid'

	DustSlipRate=2.8
	DustSlipThresh=0.1
	DustDrop=16	

	// Driver positions
	ExitPositions(0)=(X=0,Y=100,Z=50)
	ExitPositions(1)=(X=0,Y=-100,Z=50)
	ExitPositions(2)=(X=150,Y=0,Z=80)
	ExitPositions(3)=(X=-150,Y=0,Z=80)
	
	DrivePos=(X=0,Y=0,Z=0)

    OutputBrake=1
    Steering=0
    Throttle=0
    Gear=1
	StopThreshold=40

    MaxSteerAngle=6500
    MaxBrakeTorque=1

    TorqueCurve=(Points=((InVal=0,OutVal=25),(InVal=225000,OutVal=12.5),(InVal=675000,OutVal=20),(InVal=112500,OutVal=32.5),(InVal=1462500,OutVal=27.5),(InVal=1575000,OutVal=0)))
    TorqueSplit=1.0
    SteerPropGap=5000
    SteerTorque=15000
    SteerSpeed=40000

    TireMass=0.02
    ChassisMass=1
    TireRollFriction=0.6
    TireLateralFriction=0.8
    TireRollSlip=0.06
    TireLateralSlip=0.04
    TireMinSlip=0.001
    TireSlipRate=0.7
    TireSoftness=0.0
    TireAdhesion=0.0
    TireRestitution=0.0

	HandbrakeThresh=1000
	TireHandbrakeSlip=0.2
	TireHandbrakeFriction=-1.2

    SuspStiffness=65
    SuspDamping=15
    SuspHighLimit=0.10
    SuspLowLimit=-0.10
    SuspRef=0

    WheelBlurTexture(0)=Material'UR-Nas_tex.Tires.Nas_Tire_Slow'
    WheelBlurTexture(1)=Material'UR-Nas_tex.Tires.Nas_Tire_Moving'

    AirSpeed = 5000
    Health=100

    // sound
    SoundRadius=255

    UntilNextImpact=1

    //==Matt's Gear Stuff==\\
    MaxGears=5  //highest gear setting
    TorqueCurve(0)=(Points=((OutVal=2.500000),(InVal=190000.000000)))
    TorqueCurve(1)=(Points=((OutVal=2.5),(InVal=110000,OutVal=2.5),(InVal=250000,OutVal=2.5)))
    TorqueCurve(2)=(Points=((OutVal=0.32),(InVal=90000,OutVal=0.32),(InVal=380000,OutVal=2.25)))
    TorqueCurve(3)=(Points=((OutVal=0.096),(InVal=190000,OutVal=0.32),(InVal=570000,OutVal=1.61)))
    TorqueCurve(4)=(Points=((OutVal=0.032),(InVal=380000,OutVal=0.096),(InVal=760000,OutVal=0.96)))
    TorqueCurve(5)=(Points=((OutVal=0.052),(InVal=570000,OutVal=0.152),(InVal=1100000,OutVal=0.22)))
    //=====================//
}



