// ====================================================================
//  Class:  UnrealRacing.Tab_CarSettings
//  Parent: XInterface.GUITabPanel
//
//  The tab for shoosing the settings of your car
//  Matt Williams - 2003
// ====================================================================

class Tab_CarSettings extends GUITabPanel
    config(user);

var config bool bAutomaticGear;
var config class<URCar>	SelectedCar;
var class<URCar> CurrentCar;
//var class<URCar> CurrentCar;
var localized string Gears[2];
var bool bChanged;
var GUIListBox MyCarList;
var GUIScrollTextBox	MyCarDescriptionBox;
var bool bCarPrefInitialised;

var SpinnyCar			SpinnyCar; // MUST be set to null when you leave the window
var vector				SpinnyCarOffset;

function ShowPanel(bool bShow)
{
	if(!bCarPrefInitialised)
	{
		if ( Controller.OpenMenu("UnrealRacing.URLoadingCars") )
		{
			URLoadingCars(Controller.TopPage()).StartLoad(self);
		}
	}

	Super.ShowPanel(bShow);
}

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	local int i;

    Super.Initcomponent(MyController, MyOwner);

	for (i=0;i<Controls.Length;i++)
	   Controls[i].OnChange=InternalOnChange;

    MyCarList = GUIListBox(Controls[1]);
    MyCarDescriptionBox = GUIScrollTextBox(Controls[2]);

    moComboBox(Controls[0]).AddItem(Gears[0]);
    moComboBox(Controls[0]).AddItem(Gears[1]);

    moEditBox(Controls[1]).MyEditBox.bConvertSpaces = true;
	moEditBox(Controls[1]).MyEditBox.MaxWidth=16;

    if (bAutomaticGear==false)
        moComboBox(Controls[0]).SetIndex(1);
    else
        moComboBox(Controls[0]).SetIndex(0);

    // Spawn spinning car actor
	SpinnyCar = PlayerOwner().spawn(class'UnrealRacing.SpinnyCar');
	SpinnyCar.SetRotation(PlayerOwner().Rotation);
	SpinnyCar.SetDrawType(DT_Mesh);
	SpinnyCar.bPlayRandomAnims = true;
	SpinnyCar.SetDrawScale(0.9);
	SpinnyCar.SpinRate = 12000;
}

function bool InternalDraw(Canvas canvas)
{
	local vector CamPos, X, Y, Z, WX, WY, WZ;
	local rotator CamRot;
    log("Tab_CarSettings:IntermalDraw");
	if( (CurrentCar != None) && (SpinnyCar != None) )
	{
		canvas.GetCameraLocation(CamPos, CamRot);
		GetAxes(CamRot, X, Y, Z);

		if(SpinnyCar.DrawType == DT_Mesh)
		{
			GetAxes(SpinnyCar.Rotation, WX, WY, WZ);
			SpinnyCar.SetLocation(CamPos + (SpinnyCarOffset.X * X) + (SpinnyCarOffset.Y * Y) + (SpinnyCarOffset.Z * Z) + (30 * WX));
		}
		else
		{
			SpinnyCar.SetLocation(CamPos + (SpinnyCarOffset.X * X) + (SpinnyCarOffset.Y * Y) + (SpinnyCarOffset.Z * Z));
		}

		canvas.DrawActor(SpinnyCar, false, true, 90.0);
	}

	return false;
}

function InternalOnChange(GUIComponent Sender)
{
    if (!Controller.bCurMenuInitialized)
		return;

    if (Sender==Controls[4])
        bChanged=true;

	if (Sender==Controls[0])
	{
        if (moComboBox(Controls[0]).GetIndex()==1)
            bAutomaticGear=false;
        else
            bAutomaticGear=true;

        bChanged=true;
	}

	if (Sender==Controls[1])
	{
	   UpdateCurrentCar();
	   bChanged=true;
	   OnChange(Self);
	}
}

function bool InternalApply(GUIComponent Sender)
{
    local string PName;

	if (!bChanged)
		return true;

	PName = moEditBox(Controls[4]).GetText();  //get name

	PlayerOwner().UpdateURL("Name",PName, true);   //apply name

	selectedCar = class<URCar>(MyCarList.List.GetObject());

    SaveConfig();
	bChanged = false;

	return true;
}

function UpdateCurrentCar()
{
	CurrentCar = class<URCar>(MyCarList.List.GetObject());

	if (CurrentCar!=none)
	{
	   	if(CurrentCar.default.PreviewMesh != None)
		{
			SpinnyCar.SetStaticMesh(none);
			SpinnyCar.SetStaticMesh(CurrentCar.default.PreviewMesh);
			SpinnyCar.SetDrawScale(CurrentCar.default.PreviewScale);
			//SpinnyCar.SetDrawScale3D( CurrentCar.default.DrawScale3D );
			SpinnyCar.SetDrawType(DT_StaticMesh);
		}
		else
			Log("Tab_CarSettings:UpdateCurrentCar - could not find static mesh for "$CurrentCar);
	}

	MyCarDescriptionBox.SetContent( MyCarList.List.GetExtra() );
}

function InternalOnLoadINI(GUIComponent Sender, string s)
{
    if (Sender==Controls[4])
	{
		moEditBox(Sender).SetText(PlayerOwner().GetUrlOption( "Name" ));
	}
}

function OnClose(optional bool bCanceled)
{
    SaveConfig();
}

defaultproperties
{
    Begin Object class=moComboBox Name=cAutoGear
		WinWidth=0.393750
		WinHeight=0.060000
		WinLeft=0.038476
		WinTop=0.769531
		Caption="Transmission"
		INIOption="@INTERNAL"
		INIDefault="High"
		OnLoadINI=InternalOnLoadINI
		Hint="Check this to enable automatic transmission."
		CaptionWidth=0.5
		ComponentJustification=TXTA_Left
	End Object
	Controls(0)=moComboBox'cAutoGear'

	Begin Object Class=GUIListBox Name=CarList
		WinWidth=0.435000
		WinHeight=0.463633
		WinLeft=0.021641
		WinTop=0.161406
		bVisibleWhenEmpty=true
		OnChange=InternalOnChange
		StyleName="SquareButton"
		Hint="Select your car"
	End Object
	Controls(1)=GUIListBox'CarList'

	Begin Object Class=GUIScrollTextBox Name=CarDescription
		WinWidth=0.478984
		WinHeight=0.309777
		WinLeft=0.490585
		WinTop=0.689325
		CharDelay=0.0015
		EOLDelay=0.25
		bNeverFocus=true
		bAcceptsInput=false
		bVisibleWhenEmpty=true
	End Object
	Controls(2)=CarDescription

	Begin Object class=GUIImage Name=URCarBK1
		WinWidth=0.435000
		WinHeight=0.348633
		WinLeft=0.020742
		WinTop=0.650469
		Image=Material'InterfaceContent.Menu.BorderBoxD'
		ImageColor=(R=255,G=255,B=255,A=160);
		ImageRenderStyle=MSTY_Alpha
		ImageStyle=ISTY_Stretched
	End Object
	Controls(3)=GUIImage'URCarBK1'

	Begin Object class=moEditBox Name=PlayerName
		WinWidth=0.393750
		WinHeight=0.060000
		WinLeft=0.038476
		WinTop=0.689531
		Caption="Name"
		INIOption="@INTERNAL"
		INIDefault="Player"
		OnLoadINI=InternalOnLoadINI
		Hint="Changes the alias you play as."
		CaptionWidth=0.5
	End Object
	Controls(4)=moEditBox'PlayerName'

    Gears(0)="Automatic"
    Gears(1)="Manual"

    SpinnyCarOffset=(X=150,Y=60,Z=0)
    OnDraw=InternalDraw
	WinTop=0.15
	WinLeft=0
	WinWidth=1
	WinHeight=0.72
	bAcceptsInput=false
}
